#region License

// ------------------------------------------------------------------------------------------- //
// The MIT License (MIT)                                                                       //
// Copyright (c) 2012 adaptive modeling interface                                              //
//                                                                                             //
// Permission is hereby granted, free of charge, to any person obtaining a copy of this        //
// software and associated documentation files (the "Software"), to deal in the Software       //
// without restriction, including without limitation the rights to use, copy, modify, merge,   //
// publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons  //
// to whom the Software is furnished to do so, subject to the following conditions:            //
//                                                                                             //
// The above copyright notice and this permission notice shall be included in all copies or    //
// substantial portions of the Software.                                                       //
//                                                                                             //
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,         //
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR    //
// PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE   //
// FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR        //
// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER      //
// DEALINGS IN THE SOFTWARE.                                                                   //
// ------------------------------------------------------------------------------------------- //

#endregion

using Ami.Objects.Attributes;
using Ami.Objects.Core;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace Ami.Objects.Test
{
    [TestClass]
    public class CoreObjectFactory
    {
        public interface IDomain1 : IDomain
        {
        }

        [TestMethod]
        public void TestDomain1()
        {
            using (var objectModel = new ObjectModel())
            {
                var factory = objectModel.ObjectFactory;
                using (var domain = factory.CreateDomain<IDomain1>())
                {
                    Assert.IsNotNull(domain);
                    Assert.IsTrue(ReferenceEquals(domain, domain.Domain));
                }
            }
        }

        public interface IDomain2 : IDomain
        {
            [As("Domain")]
            IDomain2 MyDomain { get; }
        }

        [TestMethod]
        public void TestDomain2()
        {
            using (var objectModel = new ObjectModel())
            {
                var factory = objectModel.ObjectFactory;
                using (var domain = factory.CreateDomain<IDomain2>())
                {
                    Assert.IsNotNull(domain);
                    Assert.IsTrue(ReferenceEquals(domain, domain.MyDomain));
                }
            }
        }

        public interface IObject : Objects.IObject
        {
        }

        [TestMethod]
        public void TestNewObject()
        {
            using (var objectModel = new ObjectModel())
            {
                var factory = objectModel.ObjectFactory;
                using (var domain = factory.CreateDomain<IDomain1>())
                {
                    Assert.IsNotNull(domain);
                    using (var tobject = factory.CreateObject<IObject>(domain))
                    {
                        Assert.IsNotNull(tobject);
                        Assert.IsTrue(ReferenceEquals(domain, tobject.Domain));
                    }
                }
            }
        }

        public interface IDomainAndOwner : IDomain<IDomainAndOwner>
        {
        }

        public interface IObjectWithDomainAndOwner : IObject<IDomainAndOwner>
        {
            [AsDomain, RelationName]
            new IDomainAndOwner Owner { get; }
        }

        [TestMethod]
        public void TestDomainAndOwner()
        {
            using (var objectModel = new ObjectModel<IDomainAndOwner>())
            {
                using (var o = objectModel.Create<IObjectWithDomainAndOwner>())
                {
                    Assert.IsNotNull(o);
                    Assert.IsTrue(ReferenceEquals(o.Domain, objectModel.Domain));
                    Assert.IsNull(o.Owner);
                }
            }
        }
    }
}