#region License

// ------------------------------------------------------------------------------------------- //
// The MIT License (MIT)                                                                       //
// Copyright (c) 2012 adaptive modeling interface                                              //
//                                                                                             //
// Permission is hereby granted, free of charge, to any person obtaining a copy of this        //
// software and associated documentation files (the "Software"), to deal in the Software       //
// without restriction, including without limitation the rights to use, copy, modify, merge,   //
// publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons  //
// to whom the Software is furnished to do so, subject to the following conditions:            //
//                                                                                             //
// The above copyright notice and this permission notice shall be included in all copies or    //
// substantial portions of the Software.                                                       //
//                                                                                             //
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,         //
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR    //
// PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE   //
// FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR        //
// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER      //
// DEALINGS IN THE SOFTWARE.                                                                   //
// ------------------------------------------------------------------------------------------- //

#endregion

using System;
using System.Linq;
using Ami.Objects.Attributes;
using Ami.Objects.Core;
using Ami.Objects.Core.ReflectionModel;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace Ami.Objects.Test
{
    [TestClass]
    public class CoreReflector
    {
        public interface IErrIsIObject
        {
        }

        [TestMethod]
        public void TestIsIObject()
        {
            using (var objectModel = new ObjectModel())
            {
                var info = objectModel.Reflector[typeof (IErrIsIObject)];
                Console.Out.WriteAllInfo(objectModel.Reflector);
                Console.Out.WriteAllErrors(info);
                AssertHasOnlyInfo(info, InfoId.IsIObject);
                CommonReflectorTests(objectModel);
            }
        }

        public abstract class IErrIsInterface : AObject<IErrIsInterface, IDomain, IObject>
        {
        }

        [TestMethod]
        public void TestIsInterface()
        {
            using (var objectModel = new ObjectModel())
            {
                var info = objectModel.Reflector[typeof (IErrIsInterface)];
                Console.Out.WriteAllInfo(objectModel.Reflector);
                Console.Out.WriteAllErrors(info);
                AssertHasOnlyInfo(info, InfoId.IsInterface);
                CommonReflectorTests(objectModel);
            }
        }

        public interface OErrNameStartsWithI : IObject
        {
        }

        [TestMethod]
        public void TestNameStartsWithI()
        {
            using (var objectModel = new ObjectModel())
            {
                var info = objectModel.Reflector[typeof (OErrNameStartsWithI)];
                Console.Out.WriteAllInfo(objectModel.Reflector);
                Console.Out.WriteAllErrors(info);
                AssertHasOnlyInfo(info, InfoId.NameStartsWithI);
                CommonReflectorTests(objectModel);
            }
        }

        public interface IErrHasCommonType1 : IObject
        {
            [Attributes.Attribute]
            int Property { get; set; }
        }

        public interface IErrHasCommonType2 : IObject
        {
            [Attributes.Attribute]
            double Property { get; set; }
        }

        public interface IErrHasCommonType : IErrHasCommonType1, IErrHasCommonType2
        {
        }

        [TestMethod]
        public void TestHasPropertyType()
        {
            using (var objectModel = new ObjectModel())
            {
                var info = objectModel.Reflector[typeof (IErrHasCommonType)];
                Console.Out.WriteAllInfo(objectModel.Reflector);
                Console.Out.WriteAllErrors(info);
                AssertHasOnlyInfo(info, InfoId.HasCommonType);
                CommonReflectorTests(objectModel);
            }
        }

        public interface IErrHasKinds : IObject
        {
            double Property { get; set; }
        }

        public interface IOkHasKinds : IObject
        {
            [As("Owner"), RelationName]
            IObject Property { get; }
        }

        [TestMethod]
        public void TestHasKinds()
        {
            using (var objectModel = new ObjectModel())
            {
                var info = objectModel.Reflector[typeof (IErrHasKinds)];
                Console.Out.WriteAllInfo(objectModel.Reflector);
                Console.Out.WriteAllErrors(info);
                AssertHasOnlyInfo(info, InfoId.HasKinds);
                CommonReflectorTests(objectModel);
            }
            Console.WriteLine("----");
            using (var objectModel = new ObjectModel())
            {
                var info = objectModel.Reflector[typeof (IOkHasKinds)];
                Console.Out.WriteAllInfo(objectModel.Reflector);
                Console.Out.WriteAllErrors(info);
                AssertHasNoInfo(info);
                CommonReflectorTests(objectModel);
            }
        }

        public interface IErrHasCommonKind1 : IObject
        {
            [Attributes.Attribute]
            double Property { get; set; }
        }

        public interface IErrHasCommonKind2 : IObject
        {
            [Property]
            double Property { get; set; }
        }

        public interface IErrHasCommonKind : IErrHasCommonKind1, IErrHasCommonKind2
        {
        }

        public interface IOkHasCommonKind : IObject
        {
            [AsDomain, AsOwner, RelationName]
            IObject Value { get; }
        }

        [TestMethod]
        public void TestHasCommonKind()
        {
            using (var objectModel = new ObjectModel())
            {
                var info = objectModel.Reflector[typeof (IErrHasCommonKind)];
                Console.Out.WriteAllInfo(objectModel.Reflector);
                Console.Out.WriteAllErrors(info);
                AssertHasOnlyInfo(info, InfoId.HasCommonKind);
                CommonReflectorTests(objectModel);
            }
            Console.WriteLine("----");
            using (var objectModel = new ObjectModel())
            {
                var info = objectModel.Reflector[typeof (IOkHasCommonKind)];
                Console.Out.WriteAllInfo(objectModel.Reflector);
                Console.Out.WriteAllErrors(info);
                AssertHasNoInfo(info);
                CommonReflectorTests(objectModel);
            }
        }

        public interface IErrHasName1 : IObject
        {
            [Property]
            IObject A { get; set; }
        }

        public interface IErrHasName : IErrHasName1
        {
            [As("A"), Property]
            IObject B { get; set; }
        }

        public interface IOkHasName1 : IObject
        {
            [Property]
            IObject A { get; set; }
        }

        public interface IOkHasName : IErrHasName1
        {
            [As("A"), Property, RelationName]
            IObject B { get; set; }
        }

        [TestMethod]
        public void TestHasName()
        {
            using (var objectModel = new ObjectModel())
            {
                var info = objectModel.Reflector[typeof (IErrHasName)];
                Console.Out.WriteAllInfo(objectModel.Reflector);
                Console.Out.WriteAllErrors(info);
                AssertHasOnlyInfo(info, InfoId.HasName);
                CommonReflectorTests(objectModel);
            }
            Console.WriteLine("----");
            using (var objectModel = new ObjectModel())
            {
                var info = objectModel.Reflector[typeof (IOkHasName)];
                Console.Out.WriteAllInfo(objectModel.Reflector);
                Console.Out.WriteAllErrors(info);
                AssertHasNoInfo(info);
                CommonReflectorTests(objectModel);
            }
        }

        public interface IErrHasNoIndexer : IObject
        {
            [Property]
            double this[int index] { get; set; }
        }

        [TestMethod]
        public void TestHasNoIndexer()
        {
            using (var objectModel = new ObjectModel())
            {
                var info = objectModel.Reflector[typeof (IErrHasNoIndexer)];
                Console.Out.WriteAllInfo(objectModel.Reflector);
                Console.Out.WriteAllErrors(info);
                AssertHasOnlyInfo(info, InfoId.HasNoIndexer);
                CommonReflectorTests(objectModel);
            }
        }

        public interface IErrHasGetter : IObject
        {
            [Property]
            double Value { set; }
        }

        [TestMethod]
        public void TestHasGetter()
        {
            using (var objectModel = new ObjectModel())
            {
                var info = objectModel.Reflector[typeof (IErrHasGetter)];
                Console.Out.WriteAllInfo(objectModel.Reflector);
                Console.Out.WriteAllErrors(info);
                AssertHasOnlyInfo(info, InfoId.HasGetter);
                CommonReflectorTests(objectModel);
            }
        }

        public interface IErrReadonlyDomainRelation : IObject
        {
            [AsDomain]
            IDomain Domain { get; set; }
        }

        [TestMethod]
        public void TestReadonlyDomainRelation()
        {
            using (var objectModel = new ObjectModel())
            {
                var info = objectModel.Reflector[typeof (IErrReadonlyDomainRelation)];
                Console.Out.WriteAllInfo(objectModel.Reflector);
                Console.Out.WriteAllErrors(info);
                AssertHasOnlyInfo(info, InfoId.ReadonlyRelation);
                CommonReflectorTests(objectModel);
            }
        }

        public interface IErrSingleDomainRelationPerObject : IObject
        {
            [Domain, RelationName]
            IDomain NewDomain { get; }
        }

        public interface IOkSingleDomainRelationPerObject : IObject
        {
            [Domain, As("Domain"), RelationName]
            IDomain NewDomain { get; }
        }

        [TestMethod]
        public void TestSingleDomainRelationPerObject()
        {
            using (var objectModel = new ObjectModel())
            {
                var info = objectModel.Reflector[typeof (IErrSingleDomainRelationPerObject)];
                Console.Out.WriteAllInfo(objectModel.Reflector);
                Console.Out.WriteAllErrors(info);
                AssertHasOnlyInfo(info, InfoId.SingleRelationPerObject);
                CommonReflectorTests(objectModel);
            }
            using (var objectModel = new ObjectModel())
            {
                var info = objectModel.Reflector[typeof (IOkSingleDomainRelationPerObject)];
                Console.Out.WriteAllInfo(objectModel.Reflector);
                Console.Out.WriteAllErrors(info);
                AssertHasNoInfo(info);
                CommonReflectorTests(objectModel);
            }
        }

        public interface IErrDomainType1 : IDomain
        {
        }

        public interface IErrDomainType2 : IDomain
        {
            [Domain]
            IErrDomainType1 Domain { get; }
        }

        [TestMethod]
        public void TestDomainType()
        {
            using (var objectModel = new ObjectModel())
            {
                var info = objectModel.Reflector[typeof (IErrDomainType2)];
                Console.Out.WriteAllInfo(objectModel.Reflector);
                Console.Out.WriteAllErrors(info);
                AssertHasOnlyInfo(info, InfoId.DomainType);
                CommonReflectorTests(objectModel);
            }
        }

        public interface IO1 : IObject
        {
            [Attributes.Owner]
            IO1 Parent { get; }
        }

        public static class O1Helper
        {
        }

        [TestMethod]
        public void TestO1()
        {
            using (var objectModel = new ObjectModel())
            {
                var info = objectModel.Reflector[typeof (IO1)];
                Console.Out.WriteAllInfo(objectModel.Reflector);
                Console.Out.WriteAllErrors(info);
                Assert.IsNotNull(info);
                AssertHasNoInfo(info);
                Assert.IsTrue(typeof (O1Helper) == info.HelperType);
                CommonReflectorTests(objectModel);
            }
        }

        private static void CommonReflectorTests(IObjectModel objectModel)
        {
            foreach (var amiType in objectModel.Reflector.All)
            {
                var type = amiType;
                Assert.IsFalse(objectModel.Reflector.All.Any(t => !ReferenceEquals(t, type) && t.ReflectedType == type.ReflectedType));
            }
        }

        private static void AssertHasNotInfo(AmiInfo info, Enum id)
        {
            Assert.IsFalse(info.AllInfos.Any(i => i.Id.Equals(id)));
        }

        private static void AssertHasInfo(AmiInfo info, Enum id)
        {
            Assert.IsTrue(info.AllInfos.Any(i => i.Id.Equals(id)));
        }

        private static void AssertHasOnlyInfo(AmiInfo info, Enum id)
        {
            Assert.IsTrue(info.AllInfos.Count(i => i.Id.Equals(id)) == 1);
        }

        private static void AssertHasNoInfo(AmiInfo info)
        {
            Assert.IsTrue(info.AllInfos.Count() == 0);
        }
    }
}