#region License

// ------------------------------------------------------------------------------------------- //
// The MIT License (MIT)                                                                       //
// Copyright (c) 2012 adaptive modeling interface                                              //
//                                                                                             //
// Permission is hereby granted, free of charge, to any person obtaining a copy of this        //
// software and associated documentation files (the "Software"), to deal in the Software       //
// without restriction, including without limitation the rights to use, copy, modify, merge,   //
// publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons  //
// to whom the Software is furnished to do so, subject to the following conditions:            //
//                                                                                             //
// The above copyright notice and this permission notice shall be included in all copies or    //
// substantial portions of the Software.                                                       //
//                                                                                             //
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,         //
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR    //
// PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE   //
// FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR        //
// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER      //
// DEALINGS IN THE SOFTWARE.                                                                   //
// ------------------------------------------------------------------------------------------- //

#endregion

using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace Ami.Objects.Test
{
    [TestClass]
    public class TestReflectionHelper
    {
        public interface A1
        {
        }

        public interface A2
        {
        }

        public interface B1 : A1, A2
        {
        }

        public interface B2 : A1, A2
        {
        }

        public interface C1 : B1
        {
        }

        [TestMethod]
        public void TestDirectInterfaces()
        {
            var i = typeof (B1).DirectInterfaces();
            Assert.IsTrue(i.Count() == 2);
            Assert.IsTrue(i.Contains(typeof (A1)));
            Assert.IsTrue(i.Contains(typeof (A2)));

            i = typeof (C1).DirectInterfaces();
            Assert.IsTrue(i.Count() == 1);
            Assert.IsTrue(i.Contains(typeof (B1)));
        }

        [TestMethod]
        public void TestCommonTypes()
        {
            var list = new[] {typeof (A1), typeof (A2), typeof (B1), typeof (B1)};
            var common = list.CommonType(t => t);
            Assert.IsTrue(common != null);
            Assert.IsTrue(common == typeof (B1));

            list = new[] {typeof (A1), typeof (A1)};
            common = list.CommonType(t => t);
            Assert.IsTrue(common != null);
            Assert.IsTrue(common == typeof (A1));

            list = new[] {typeof (A1), typeof (A2), typeof (B1), typeof (B1), typeof (B2)};
            common = list.CommonType(t => t);
            Assert.IsFalse(common != null);
        }
    }
}