#region License

// ------------------------------------------------------------------------------------------- //
// The MIT License (MIT)                                                                       //
// Copyright (c) 2012 adaptive modeling interface                                              //
//                                                                                             //
// Permission is hereby granted, free of charge, to any person obtaining a copy of this        //
// software and associated documentation files (the "Software"), to deal in the Software       //
// without restriction, including without limitation the rights to use, copy, modify, merge,   //
// publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons  //
// to whom the Software is furnished to do so, subject to the following conditions:            //
//                                                                                             //
// The above copyright notice and this permission notice shall be included in all copies or    //
// substantial portions of the Software.                                                       //
//                                                                                             //
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,         //
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR    //
// PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE   //
// FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR        //
// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER      //
// DEALINGS IN THE SOFTWARE.                                                                   //
// ------------------------------------------------------------------------------------------- //

#endregion

using System;
using Ami.Objects.Core.ReflectionModel;

namespace Ami.Objects.Attributes
{
    /// <summary>
    /// Marks a property value has being owned by the <see cref="IObject"/> defining it.
    /// </summary>
    [AttributeUsage(AttributeTargets.Property, AllowMultiple = false, Inherited = false)]
    public class PropertyAttribute : RelationKindAttribute
    {
        /// <summary>
        /// <see cref="PropertyAttribute"/> kind.
        /// </summary>
        public interface IKind
        {
        }

        /// <summary>
        /// Order used by <see cref="AmiRelation.Comparer"/>.
        /// </summary>
        public override int Order
        {
            get { return int.MinValue + 2; }
        }

        /// <summary>
        /// Get the kind of this attribute.
        /// This must return an interface.
        /// </summary>
        protected override Type GetKind
        {
            get { return typeof (IKind); }
        }

        /// <summary>
        /// Implements an <see cref="PropertyAttribute"/>.
        /// </summary>
        /// <param name="relation">Relation to implement.</param>
        public override string Implement(AmiRelation relation)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Create the corresponding relation instanciation.
        /// The text writer is like:
        /// <code>
        /// namespace N
        /// {
        ///     public class C
        ///     {
        ///         void CreateRelations()
        ///         {
        /// </code>
        /// </summary>
        /// <param name="relation">Relation to create.</param>
        /// <param name="domain">True if the relation to create is the object's domain relation.</param>
        public override string CreateRelation(AmiRelation relation, bool domain)
        {
            throw new NotImplementedException();
        }
    }
}