#region License

// ------------------------------------------------------------------------------------------- //
// The MIT License (MIT)                                                                       //
// Copyright (c) 2012 adaptive modeling interface                                              //
//                                                                                             //
// Permission is hereby granted, free of charge, to any person obtaining a copy of this        //
// software and associated documentation files (the "Software"), to deal in the Software       //
// without restriction, including without limitation the rights to use, copy, modify, merge,   //
// publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons  //
// to whom the Software is furnished to do so, subject to the following conditions:            //
//                                                                                             //
// The above copyright notice and this permission notice shall be included in all copies or    //
// substantial portions of the Software.                                                       //
//                                                                                             //
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,         //
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR    //
// PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE   //
// FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR        //
// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER      //
// DEALINGS IN THE SOFTWARE.                                                                   //
// ------------------------------------------------------------------------------------------- //

#endregion

namespace Ami.Objects.Core
{
    /// <summary>
    /// <see cref="IDomain"/> abstract implementation.
    /// </summary>
    /// <typeparam name="TObject">Final domain type.</typeparam>
    /// <typeparam name="TDomain">Declared domain type.</typeparam>
    /// <typeparam name="TOwner">Owner type.</typeparam>
    public abstract class ADomain<TObject, TDomain, TOwner> : AObject<TObject, TDomain, TOwner>, IDomain<TDomain> where TObject : ADomain<TObject, TDomain, TOwner> where TDomain : class, IDomain where TOwner : class, IObject
    {
        /// <summary>
        /// Initializes a new instance of <see cref="ADomain{TObject, TDomain, TOwner}"/> with the given <paramref name="objectModel"/>.
        /// </summary>
        protected ADomain(IObjectModel objectModel)
        {
            ObjectModel = objectModel;
        }

        /// <summary>
        /// Get the implementation type of this <see cref="IObject"/>.
        /// </summary>
        public IObjectModel ObjectModel { get; private set; }

        /// <summary>
        /// Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources.
        /// </summary>
        /// <filterpriority>2</filterpriority>
        public override void Dispose()
        {
            base.Dispose();
            ObjectModel = null;
        }
    }
}