#region License

// ------------------------------------------------------------------------------------------- //
// The MIT License (MIT)                                                                       //
// Copyright (c) 2012 adaptive modeling interface                                              //
//                                                                                             //
// Permission is hereby granted, free of charge, to any person obtaining a copy of this        //
// software and associated documentation files (the "Software"), to deal in the Software       //
// without restriction, including without limitation the rights to use, copy, modify, merge,   //
// publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons  //
// to whom the Software is furnished to do so, subject to the following conditions:            //
//                                                                                             //
// The above copyright notice and this permission notice shall be included in all copies or    //
// substantial portions of the Software.                                                       //
//                                                                                             //
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,         //
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR    //
// PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE   //
// FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR        //
// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER      //
// DEALINGS IN THE SOFTWARE.                                                                   //
// ------------------------------------------------------------------------------------------- //

#endregion

namespace Ami.Objects.Core
{
    /// <summary>
    /// <see cref="IObjectModel"/> abstract implementation.
    /// </summary>
    public abstract class AObjectModel : IObjectModel
    {
        private readonly object lockClassFactory = new object();
        private readonly object lockObjectFactory = new object();
        private readonly object lockReflector = new object();

        private IClassFactory classFactory;

        /// <summary>
        /// Creates the <see cref="ClassFactory"/> when required.
        /// </summary>
        /// <returns>A new <see cref="IClassFactory"/> instance.</returns>
        protected abstract IClassFactory CreateClassFactory();

        /// <summary>
        /// <see cref="IClassFactory"/> associated with this <see cref="IObjectModel"/>.
        /// </summary>
        public IClassFactory ClassFactory
        {
            get { lock (lockClassFactory) return classFactory ?? (classFactory = CreateClassFactory()); }
        }

        private IObjectFactory objectFactory;

        /// <summary>
        /// Creates the <see cref="ObjectFactory"/> when required.
        /// </summary>
        /// <returns>A new <see cref="IObjectFactory"/> instance.</returns>
        protected abstract IObjectFactory CreateObjectFactory();

        /// <summary>
        /// <see cref="IObjectFactory"/> associated with this <see cref="IObjectModel"/>.
        /// </summary>
        public IObjectFactory ObjectFactory
        {
            get { lock (lockObjectFactory) return objectFactory ?? (objectFactory = CreateObjectFactory()); }
        }

        private IReflector reflector;

        /// <summary>
        /// Creates the <see cref="Reflector"/> when required.
        /// </summary>
        /// <returns>A new <see cref="IReflector"/> instance.</returns>
        protected abstract IReflector CreateReflector();

        /// <summary>
        /// <see cref="IReflector"/> associated with this <see cref="IObjectModel"/>.
        /// </summary>
        public IReflector Reflector
        {
            get { lock (lockReflector) return reflector ?? (reflector = CreateReflector()); }
        }

        /// <summary>
        /// Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources.
        /// </summary>
        /// <filterpriority>2</filterpriority>
        public virtual void Dispose()
        {
            ObjectHelper.TryDispose(ref classFactory);
            ObjectHelper.TryDispose(ref objectFactory);
            ObjectHelper.TryDispose(ref reflector);
        }
    }
}