#region License

// ------------------------------------------------------------------------------------------- //
// The MIT License (MIT)                                                                       //
// Copyright (c) 2012 adaptive modeling interface                                              //
//                                                                                             //
// Permission is hereby granted, free of charge, to any person obtaining a copy of this        //
// software and associated documentation files (the "Software"), to deal in the Software       //
// without restriction, including without limitation the rights to use, copy, modify, merge,   //
// publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons  //
// to whom the Software is furnished to do so, subject to the following conditions:            //
//                                                                                             //
// The above copyright notice and this permission notice shall be included in all copies or    //
// substantial portions of the Software.                                                       //
//                                                                                             //
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,         //
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR    //
// PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE   //
// FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR        //
// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER      //
// DEALINGS IN THE SOFTWARE.                                                                   //
// ------------------------------------------------------------------------------------------- //

#endregion

namespace Ami.Objects.Core.ReflectionModel
{
    /// <summary>
    /// Enumerates all error and warning codes for ReflectionModel.
    /// </summary>
    public enum InfoId
    {
        /// <summary>
        /// Error when <see cref="AmiType.ReflectedType"/> is not an <see cref="IObject"/>.
        /// </summary>
        [Error] IsIObject,

        /// <summary>
        /// Error when <see cref="AmiType.ReflectedType"/> is not an interface.
        /// </summary>
        [Error] IsInterface,

        /// <summary>
        /// Error when <see cref="AmiType.ReflectedType"/> name dosen't start with 'I'.
        /// </summary>
        [Error] NameStartsWithI,

        /// <summary>
        /// Error when <see cref="AmiRelation.RelationType"/> is null.
        /// </summary>
        [Error] HasCommonType,

        /// <summary>
        /// Error when <see cref="AmiRelation.Kinds"/> is empty.
        /// </summary>
        [Error] HasKinds,

        /// <summary>
        /// Error when <see cref="AmiRelation.Kind"/> has not been found.
        /// </summary>
        [Error] HasCommonKind,

        /// <summary>
        /// Error when <see cref="AmiRelation.Name"/> has not been found.
        /// </summary>
        [Error] HasName,

        /// <summary>
        /// Error when <see cref="AmiProperty.ReflectedProperties"/> have indexers.
        /// </summary>
        [Error] HasNoIndexer,

        /// <summary>
        /// Error when <see cref="AmiProperty.ReflectedProperties"/> have no getter.
        /// </summary>
        [Error] HasGetter,

        /// <summary>
        /// Error when some <see cref="AmiProperty.ReflectedProperties"/> have a setter.
        /// </summary>
        [Error] ReadonlyRelation,

        /// <summary>
        /// Error when some an <see cref="AmiType"/> have more than one relation of this kind.
        /// </summary>
        [Error] SingleRelationPerObject,

        /// <summary>
        /// Error when domain type specified on <see cref="IDomain"/> is not valid.
        /// </summary>
        [Error] DomainType,
    }
}