#region License

// ------------------------------------------------------------------------------------------- //
// The MIT License (MIT)                                                                       //
// Copyright (c) 2012 adaptive modeling interface                                              //
//                                                                                             //
// Permission is hereby granted, free of charge, to any person obtaining a copy of this        //
// software and associated documentation files (the "Software"), to deal in the Software       //
// without restriction, including without limitation the rights to use, copy, modify, merge,   //
// publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons  //
// to whom the Software is furnished to do so, subject to the following conditions:            //
//                                                                                             //
// The above copyright notice and this permission notice shall be included in all copies or    //
// substantial portions of the Software.                                                       //
//                                                                                             //
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,         //
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR    //
// PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE   //
// FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR        //
// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER      //
// DEALINGS IN THE SOFTWARE.                                                                   //
// ------------------------------------------------------------------------------------------- //

#endregion

using Ami.Objects.Attributes;

namespace Ami.Objects
{
    /// <summary>
    /// <see cref="IDomain"/> specifies <see cref="IObject"/>'s origin.
    /// </summary>
    [Abstract]
    public interface IDomain : IObject
    {
        /// <summary>
        /// Get the <see cref="IObjectModel"/> of this <see cref="IObject"/>.
        /// </summary>
        [NotAmi]
        IObjectModel ObjectModel { get; }
    }

    /// <summary>
    /// Typed <see cref="IDomain"/>.
    /// </summary>
    /// <typeparam name="TDomain">Domain type.</typeparam>
    [Abstract]
    public interface IDomain<out TDomain> : IDomain, IObject<TDomain> where TDomain : IDomain
    {
    }

    /// <summary>
    /// Typed <see cref="IDomain"/>.
    /// <remarks>Domains cannot be owned.</remarks>
    /// </summary>
    /// <typeparam name="TDomain">Domain type.</typeparam>
    /// <typeparam name="TOwner">Object's owner type in this domain.</typeparam>
    [Abstract]
    public interface IDomain<out TDomain, out TOwner> : IDomain<TDomain>, IObject<TDomain, TOwner> where TDomain : IDomain where TOwner : IObject
    {
    }

    /// <summary>
    /// <see cref="IDomain"/> extension.
    /// </summary>
    public static class DomainHelper
    {
        /// <summary>
        /// Create a new instance of <typeparamref name="T"/>.
        /// </summary>
        /// <param name="domain"><see cref="IObject"/>'s domain.</param>
        /// <typeparam name="T">Type of object to create, must be an interface.</typeparam>
        /// <returns>A new instance of <typeparamref name="T"/>.</returns>
        public static T Create<T>(this IDomain domain) where T : IObject
        {
            return domain.ObjectModel.Create<T>(domain);
        }
    }
}