#region License

// ------------------------------------------------------------------------------------------- //
// The MIT License (MIT)                                                                       //
// Copyright (c) 2012 adaptive modeling interface                                              //
//                                                                                             //
// Permission is hereby granted, free of charge, to any person obtaining a copy of this        //
// software and associated documentation files (the "Software"), to deal in the Software       //
// without restriction, including without limitation the rights to use, copy, modify, merge,   //
// publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons  //
// to whom the Software is furnished to do so, subject to the following conditions:            //
//                                                                                             //
// The above copyright notice and this permission notice shall be included in all copies or    //
// substantial portions of the Software.                                                       //
//                                                                                             //
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,         //
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR    //
// PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE   //
// FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR        //
// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER      //
// DEALINGS IN THE SOFTWARE.                                                                   //
// ------------------------------------------------------------------------------------------- //

#endregion

using Ami.Objects.Relations.Values;

namespace Ami.Objects.Relations
{
    /// <summary>
    /// See: <see cref="IAttribute.AttributeChanged"/>.
    /// </summary>
    /// <param name="relation"><see cref="IAttribute"/>.</param>
    /// <param name="previousValue">Previous value.</param>
    /// <param name="newValue">New value.</param>
    public delegate void AttributeChanged(IAttribute relation, object previousValue, object newValue);

    /// <summary>
    /// See: <see cref="IAttribute.AttributeChanged"/>.
    /// </summary>
    /// <param name="relation"><see cref="IAttribute"/>.</param>
    /// <param name="previousValue">Previous value.</param>
    /// <param name="newValue">New value.</param>
    public delegate void AttributeChanged<TValue>(IAttribute<TValue> relation, TValue previousValue, TValue newValue);

    /// <summary>
    /// See: <see cref="IAttribute.AttributeChanged"/>.
    /// </summary>
    /// <param name="relation"><see cref="IAttribute"/>.</param>
    /// <param name="previousValue">Previous value.</param>
    /// <param name="newValue">New value.</param>
    public delegate void AttributeChanged<TValue, TRelationOwner>(IAttribute<TValue, TRelationOwner> relation, TValue previousValue, TValue newValue) where TRelationOwner : IObject;

    /// <summary>
    /// A <see cref="IRelation"/> as attribute.
    /// </summary>
    public interface IAttribute : IValuedRelation
    {
        /// <summary>
        /// Relation value.
        /// </summary>
        new IAttributeValue Value { get; set; }

        /// <summary>
        /// Occurs when <see cref="Value"/> has changed.
        /// </summary>
        event AttributeChanged AttributeChanged;
    }

    /// <summary>
    /// A typed <see cref="IAttribute"/> relation.
    /// </summary>
    /// <typeparam name="TValue">Value type.</typeparam>
    public interface IAttribute<TValue> : IAttribute, IValuedRelation<TValue>
    {
        /// <summary>
        /// Relation value.
        /// </summary>
        new IAttributeValue<TValue> Value { get; set; }

        /// <summary>
        /// Occurs when <see cref="Value"/> has changed.
        /// </summary>
        new event AttributeChanged<TValue> AttributeChanged;
    }

    /// <summary>
    /// A typed <see cref="IAttribute"/> relation.
    /// </summary>
    /// <typeparam name="TValue">Value type.</typeparam>
    /// <typeparam name="TRelationOwner">Relation onwer type.</typeparam>
    public interface IAttribute<TValue, TRelationOwner> : IAttribute<TValue>, IValuedRelation<TValue, TRelationOwner> where TRelationOwner : IObject
    {
        /// <summary>
        /// Relation value.
        /// </summary>
        new IAttributeValue<TValue, TRelationOwner> Value { get; set; }

        /// <summary>
        /// Occurs when <see cref="Value"/> has changed.
        /// </summary>
        new event AttributeChanged<TValue, TRelationOwner> AttributeChanged;
    }
}