#region License

// ------------------------------------------------------------------------------------------- //
// The MIT License (MIT)                                                                       //
// Copyright (c) 2012 adaptive modeling interface                                              //
//                                                                                             //
// Permission is hereby granted, free of charge, to any person obtaining a copy of this        //
// software and associated documentation files (the "Software"), to deal in the Software       //
// without restriction, including without limitation the rights to use, copy, modify, merge,   //
// publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons  //
// to whom the Software is furnished to do so, subject to the following conditions:            //
//                                                                                             //
// The above copyright notice and this permission notice shall be included in all copies or    //
// substantial portions of the Software.                                                       //
//                                                                                             //
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,         //
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR    //
// PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE   //
// FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR        //
// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER      //
// DEALINGS IN THE SOFTWARE.                                                                   //
// ------------------------------------------------------------------------------------------- //

#endregion

namespace Ami.Objects.Relations
{
    /// <summary>
    /// See: <see cref="IOwnerRelation.ObjectOwnerChanged"/>.
    /// </summary>
    /// <param name="relation"><see cref="IOwnerRelation"/>.</param>
    /// <param name="previousOwner">Previous value.</param>
    /// <param name="newOwner">New value.</param>
    public delegate void ObjectOwnerChanged(IOwnerRelation relation, IObject previousOwner, IObject newOwner);

    /// <summary>
    /// See: <see cref="IOwnerRelation{TObjectOwner}"/>.
    /// </summary>
    /// <typeparam name="TObjectOwner"><see cref="IOwnerRelation{TObjectOwner}"/>.</typeparam>
    /// <param name="relation"><see cref="IOwnerRelation"/>.</param>
    /// <param name="previousOwner">Previous value.</param>
    /// <param name="newOwner">New value.</param>
    public delegate void ObjectOwnerChanged<TObjectOwner>(IOwnerRelation<TObjectOwner> relation, TObjectOwner previousOwner, TObjectOwner newOwner) where TObjectOwner : IObject;

    /// <summary>
    /// See: <see cref="IOwnerRelation{TObjectOwner, TRelationOwner}"/>.
    /// </summary>
    /// <typeparam name="TObjectOwner"><see cref="IOwnerRelation{TObjectOwner, TRelationOwner}"/>.</typeparam>
    /// <typeparam name="TRelationOwner"><see cref="IOwnerRelation{TObjectOwner, TRelationOwner}"/>.</typeparam>
    /// <param name="relation"><see cref="IOwnerRelation"/>.</param>
    /// <param name="previousOwner">Previous value.</param>
    /// <param name="newOwner">New value.</param>
    public delegate void ObjectOwnerChanged<TObjectOwner, in TRelationOwner>(IOwnerRelation<TObjectOwner, TRelationOwner> relation, TObjectOwner previousOwner, TObjectOwner newOwner) where TObjectOwner : IObject where TRelationOwner : IObject;

    /// <summary>
    /// <see cref="IObject.Owner"/> relation.
    /// </summary>
    public interface IOwnerRelation : IRelation
    {
        /// <summary>
        /// <see cref="IObject"/> owned by <see cref="IRelation.RelationOwner"/>.
        /// This property is equivalent to <see cref="IRelation.RelationOwner"/>.
        /// </summary>
        IObject Owned { get; }

        /// <summary>
        /// <see cref="IObject.Owner"/> value.
        /// </summary>
        IObject ObjectOwner { get; set; }

        /// <summary>
        /// Occurs when <see cref="ObjectOwner"/> has changed.
        /// </summary>
        event ObjectOwnerChanged ObjectOwnerChanged;
    }

    /// <summary>
    /// Typed <see cref="IOwnerRelation"/>.
    /// </summary>
    /// <typeparam name="TObjectOwner"><see cref="IObject"/>'s owner type.</typeparam>
    public interface IOwnerRelation<TObjectOwner> : IOwnerRelation where TObjectOwner : IObject
    {
        /// <summary>
        /// Typed <see cref="IOwnerRelation.ObjectOwner"/>.
        /// </summary>
        new TObjectOwner ObjectOwner { get; set; }

        /// <summary>
        /// Occurs when <see cref="ObjectOwner"/> has changed.
        /// </summary>
        new event ObjectOwnerChanged<TObjectOwner> ObjectOwnerChanged;
    }

    /// <summary>
    /// Typed <see cref="IOwnerRelation"/>.
    /// </summary>
    /// <typeparam name="TObjectOwner"><see cref="IObject"/>'s owner type.</typeparam>
    /// <typeparam name="TRelationOwner">Relation owner type.</typeparam>
    public interface IOwnerRelation<TObjectOwner, out TRelationOwner> : IOwnerRelation<TObjectOwner>, IRelation<TRelationOwner> where TObjectOwner : IObject where TRelationOwner : IObject
    {
        /// <summary>
        /// <see cref="IObject"/> owned by <see cref="IRelation.RelationOwner"/>.
        /// This property is equivalent to <see cref="IRelation{TRelationOwner}.RelationOwner"/>.
        /// </summary>
        new TRelationOwner Owned { get; }

        /// <summary>
        /// Occurs when <see cref="IOwnerRelation{TObjectOwner}.ObjectOwner"/> has changed.
        /// </summary>
        new event ObjectOwnerChanged<TObjectOwner, TRelationOwner> ObjectOwnerChanged;
    }
}