#region License

// ------------------------------------------------------------------------------------------- //
// The MIT License (MIT)                                                                       //
// Copyright (c) 2012 adaptive modeling interface                                              //
//                                                                                             //
// Permission is hereby granted, free of charge, to any person obtaining a copy of this        //
// software and associated documentation files (the "Software"), to deal in the Software       //
// without restriction, including without limitation the rights to use, copy, modify, merge,   //
// publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons  //
// to whom the Software is furnished to do so, subject to the following conditions:            //
//                                                                                             //
// The above copyright notice and this permission notice shall be included in all copies or    //
// substantial portions of the Software.                                                       //
//                                                                                             //
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,         //
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR    //
// PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE   //
// FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR        //
// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER      //
// DEALINGS IN THE SOFTWARE.                                                                   //
// ------------------------------------------------------------------------------------------- //

#endregion

using System;

namespace Ami.Objects.Relations
{
    /// <summary>
    /// A relation of a <see cref="IObject"/>.
    /// </summary>
    public interface IRelation : IDisposable
    {
        /// <summary>
        /// <see cref="IRelation"/>'s name.
        /// </summary>
        string Name { get; }

        /// <summary>
        /// <see cref="IObject"/> owning this relation.
        /// </summary>
        IObject RelationOwner { get; }

        /// <summary>
        /// Initializes this relation.
        /// </summary>
        void Initialize();
    }

    /// <summary>
    /// Type <see cref="IRelation"/>.
    /// </summary>
    /// <typeparam name="TRelationOwner">Owner type.</typeparam>
    public interface IRelation<out TRelationOwner> : IRelation where TRelationOwner : IObject
    {
        /// <summary>
        /// <typeparam name="TRelationOwner"/> owning this relation.
        /// </summary>
        new TRelationOwner RelationOwner { get; }
    }
}