#region License

// ------------------------------------------------------------------------------------------- //
// The MIT License (MIT)                                                                       //
// Copyright (c) 2012 adaptive modeling interface                                              //
//                                                                                             //
// Permission is hereby granted, free of charge, to any person obtaining a copy of this        //
// software and associated documentation files (the "Software"), to deal in the Software       //
// without restriction, including without limitation the rights to use, copy, modify, merge,   //
// publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons  //
// to whom the Software is furnished to do so, subject to the following conditions:            //
//                                                                                             //
// The above copyright notice and this permission notice shall be included in all copies or    //
// substantial portions of the Software.                                                       //
//                                                                                             //
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,         //
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR    //
// PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE   //
// FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR        //
// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER      //
// DEALINGS IN THE SOFTWARE.                                                                   //
// ------------------------------------------------------------------------------------------- //

#endregion

namespace Ami.Objects.Relations.Values
{
    /// <summary>
    /// Value for an attribute.
    /// </summary>
    public interface IAttributeValue : IValue
    {
        /// <summary>
        /// <see cref="IAttributeValue"/>'s relation.
        /// </summary>
        new IAttribute Relation { get; }
    }

    /// <summary>
    /// Typed <see cref="IAttributeValue"/>.
    /// </summary>
    /// <typeparam name="T">Attribute type.</typeparam>
    public interface IAttributeValue<T> : IAttributeValue, IValue<T>
    {
        /// <summary>
        /// <see cref="IAttributeValue{T}"/>'s relation.
        /// </summary>
        new IAttribute<T> Relation { get; }
    }

    /// <summary>
    /// Typed <see cref="IAttributeValue"/>.
    /// </summary>
    /// <typeparam name="T">Attribute type.</typeparam>
    /// <typeparam name="TOwner">Owner type.</typeparam>
    public interface IAttributeValue<T, TOwner> : IAttributeValue<T>, IValue<T, TOwner> where TOwner : IObject
    {
        /// <summary>
        /// <see cref="IAttributeValue{T, TOnwer}"/>'s relation.
        /// </summary>
        new IAttribute<T, TOwner> Relation { get; }
    }
}