#region License

// ------------------------------------------------------------------------------------------- //
// The MIT License (MIT)                                                                       //
// Copyright (c) 2012 adaptive modeling interface                                              //
//                                                                                             //
// Permission is hereby granted, free of charge, to any person obtaining a copy of this        //
// software and associated documentation files (the "Software"), to deal in the Software       //
// without restriction, including without limitation the rights to use, copy, modify, merge,   //
// publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons  //
// to whom the Software is furnished to do so, subject to the following conditions:            //
//                                                                                             //
// The above copyright notice and this permission notice shall be included in all copies or    //
// substantial portions of the Software.                                                       //
//                                                                                             //
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,         //
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR    //
// PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE   //
// FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR        //
// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER      //
// DEALINGS IN THE SOFTWARE.                                                                   //
// ------------------------------------------------------------------------------------------- //

#endregion

using System;

namespace Ami.Objects.Relations.Values
{
    /// <summary>
    /// Value of <see cref="IValuedRelation"/>
    /// </summary>
    public interface IValue : IDisposable
    {
        /// <summary>
        /// <see cref="IValue"/>'s relation.
        /// </summary>
        IValuedRelation Relation { get; }

        /// <summary>
        /// Value of the <see cref="IValuedRelation"/>.
        /// </summary>
        object Value { get; }
    }

    /// <summary>
    /// Typed <see cref="IValue"/>.
    /// </summary>
    /// <typeparam name="T">Value type.</typeparam>
    public interface IValue<T> : IValue
    {
        /// <summary>
        /// Typed <see cref="IValue"/>'relation.
        /// </summary>
        new IValuedRelation<T> Relation { get; }

        /// <summary>
        /// Value of the <see cref="IValuedRelation"/>.
        /// </summary>
        new T Value { get; }
    }

    /// <summary>
    /// 
    /// </summary>
    /// <typeparam name="T"></typeparam>
    /// <typeparam name="TOnwer"></typeparam>
    public interface IValue<T, TOnwer> : IValue<T> where TOnwer : IObject
    {
        /// <summary>
        /// Typed <see cref="IValue"/>'s relation.
        /// </summary>
        new IValuedRelation<T, TOnwer> Relation { get; }
    }

    /// <summary>
    /// <see cref="IValue"/> extensions.
    /// </summary>
    public static class ValueHelper
    {
        /// <summary>
        /// Get the owner of the given <paramref name="value"/>.
        /// </summary>
        /// <param name="value"><see cref="IValue"/>.</param>
        /// <returns><see cref="IValue"/>'s owner</returns>
        public static IObject Onwer(this IValue value)
        {
            return value.Relation.RelationOwner;
        }
    }
}